"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const schema_1 = require("./src/schema");
class MongoLocksError extends Error {
    constructor(message) {
        super(message);
    }
}
exports.MongoLocksError = MongoLocksError;
const makeLockId = array => Array.prototype.slice.apply(array).join(":");
class LockManager {
    constructor() {
        this.Locks = null;
        this.MongoLocksError = MongoLocksError;
        this.noop = () => __awaiter(this, void 0, void 0, function* () { });
    }
    init(connection, options) {
        options = options || {};
        this.Locks = connection.model(options.collection || 'Locks', schema_1.default);
    }
    lock(...actions) {
        if (!this.Locks) {
            throw new MongoLocksError("You must initialize mongo-locks with a mongoose connection");
        }
        const lockId = makeLockId(actions);
        const l = new this.Locks();
        l.action = lockId;
        return l.save().then(() => () => this.Locks.deleteOne({ action: lockId }).exec());
    }
    refresh(...actions) {
        if (!this.Locks) {
            throw new MongoLocksError("You must initialize mongo-locks with a mongoose connection");
        }
        return this.Locks.updateOne({ action: makeLockId(actions) }, { $set: { refreshedAt: Date.now() } }).exec();
    }
    free(...actions) {
        if (!this.Locks) {
            throw new MongoLocksError("You must initialize mongo-locks with a mongoose connection");
        }
        return this.Locks.deleteOne({ action: makeLockId(actions) }).exec();
    }
}
exports.default = new LockManager();
